/*
   Module to allow translation from characters to SGML-entities and vice-versa,
   used e.g. in HTML

program:   sgml_entities.rex
type:      Object REXX, REXXSAA 6.0
purpose:   implements functions and classes for x-lating chars <--> SGML-entities (HTML)
version:   1.2.1
date:      1997-04-15
changed:   1997-08-26, ---rgf, CHANGESTR() is called only, if there is something to change
           1999-09-11, ---rgf, added iso-8859-1 ("latin1") plus usage of charset-alias-names
           2005-05-29, ---rgf, renamed ".cmd" to ".rex"

author:    Rony G. Flatscher
           Rony.Flatscher@wu-wien.ac.at
           (Wirtschaftsuniversitaet Wien, University of Economics and Business
           Administration, Vienna/Austria/Europe)

needs:     NLS_UTIL.rex

usage:     call or require this module, which will set up tables for translating
           characters according to a code-page to the appropriate SGML-entities as
           they are used in HTML-files (being based on SGML); thereafter the following
           two routines are available:

             CPString2SGMLEntity( string [, [CodePage] [, "Reverse" ] ] )
                ... returns a string in which characters are translated according
                    to the given codepage (defaults to NLS_UTIL' default codepage);
                    if the third argument "R" is given then translation occurs from
                    SGML-entities to the appropriate characters according to the given
                    codepage

             CPFile2SGMLEntity( filename [, [CodePage] [, "Reverse" ] ] )
                ... works on the contents of the file denoted by "filename"; for
                    its work it uses CPString2SGMLEntity(), hence the second and
                    third argument are the same as above

             The following codepages (charsets) are pre-defined (each row contains
             all of the available alias-names for that particular codepage)

                    437 ibm437 437us  ... codepage 437
                    850 ibm850 850de  ... codepage 850
                    819 ibm819 iso8859-1 iso_8859-1 iso88591 latin1  ... codepage 819
                    0                 ... default codepage (according to process' codepage
                                          under OS/2, "latin1" else, e.g. for Linux


comments:  prepared for the "8th International Rexx Symposium 1997", April 1997, Heidelberg/Germany


All rights reserved and copyrighted 1997 by the author,
no guarantee that it works without errors, etc. etc.

You are granted the right to use this module under the condition that you don't charge money for this module (as you didn't write
it in the first place) or modules directly derived from this module, that you document the original author (to give appropriate
credit) with the original name of the module and that you make the unaltered, original source-code of this module available on
demand.  If that holds, you may even bundle this module (either in source or compiled form) with commercial software.

If you find an error, please send me the description (preferably a *very* short example);
I'll try to fix it and re-release it to the net.

!!!!!!
If you extend the definitions with the codepage of *your* country, *please* send me those
definitions and I will extend this module and re-release it to the net.
!!!!!!
*/



/* initialize, setup CP 850 (Western Europe),  437 (US) and ISO-8859-1 (latin1) */

/* Codepage 850 (Western Europe)        */
cp850tab = .table ~ new                            /* create a table */

/*       VALUE sgml-entity: |  INDEX in form of the appr. CP-character     */
cp850tab ~ PUT( "&Ccedil;"  ,  D2C( 128 ) )
cp850tab ~ PUT( "&uuml;"    ,  D2C( 129 ) )     /* e.g. German umlaut-"u"       */
cp850tab ~ PUT( "&eacute;"  ,  D2C( 130 ) )
cp850tab ~ PUT( "&acirc;"   ,  D2C( 131 ) )
cp850tab ~ PUT( "&auml;"    ,  D2C( 132 ) )
cp850tab ~ PUT( "&agrave;"  ,  D2C( 133 ) )     /* e.g. french "a" with accent grave */
cp850tab ~ PUT( "&aring;"   ,  D2C( 134 ) )
cp850tab ~ PUT( "&ccedil;"  ,  D2C( 135 ) )
cp850tab ~ PUT( "&ecirc;"   ,  D2C( 136 ) )
cp850tab ~ PUT( "&euml;"    ,  D2C( 137 ) )
cp850tab ~ PUT( "&egrave;"  ,  D2C( 138 ) )
cp850tab ~ PUT( "&iuml;"    ,  D2C( 139 ) )
cp850tab ~ PUT( "&icirc;"   ,  D2C( 140 ) )
cp850tab ~ PUT( "&igrave;"  ,  D2C( 141 ) )
cp850tab ~ PUT( "&Auml;"    ,  D2C( 142 ) )
cp850tab ~ PUT( "&Aring;"   ,  D2C( 143 ) )     /* e.g. nordic "A" with a ring on top */
cp850tab ~ PUT( "&Eacute;"  ,  D2C( 144 ) )
cp850tab ~ PUT( "&aelig;"   ,  D2C( 145 ) )
cp850tab ~ PUT( "&AElig;"   ,  D2C( 146 ) )
cp850tab ~ PUT( "&ocirc;"   ,  D2C( 147 ) )
cp850tab ~ PUT( "&ouml;"    ,  D2C( 148 ) )
cp850tab ~ PUT( "&ograve;"  ,  D2C( 149 ) )
cp850tab ~ PUT( "&ucirc;"   ,  D2C( 150 ) )
cp850tab ~ PUT( "&ugrave;"  ,  D2C( 151 ) )
cp850tab ~ PUT( "&yuml;"    ,  D2C( 152 ) )
cp850tab ~ PUT( "&Ouml;"    ,  D2C( 153 ) )
cp850tab ~ PUT( "&Uuml;"    ,  D2C( 154 ) )
cp850tab ~ PUT( "&oslash;"  ,  D2C( 155 ) )
cp850tab ~ PUT( "&pound;"   ,  D2C( 156 ) )
cp850tab ~ PUT( "&Oslash;"  ,  D2C( 157 ) )
cp850tab ~ PUT( "&times;"   ,  D2C( 158 ) )
cp850tab ~ PUT( "&aacute;"  ,  D2C( 160 ) )
cp850tab ~ PUT( "&iacute;"  ,  D2C( 161 ) )
cp850tab ~ PUT( "&oacute;"  ,  D2C( 162 ) )
cp850tab ~ PUT( "&uacute;"  ,  D2C( 163 ) )
cp850tab ~ PUT( "&ntilde;"  ,  D2C( 164 ) )
cp850tab ~ PUT( "&Ntilde;"  ,  D2C( 165 ) )
cp850tab ~ PUT( "&ordf;"    ,  D2C( 166 ) )
cp850tab ~ PUT( "&ordm;"    ,  D2C( 167 ) )
cp850tab ~ PUT( "&iquest;"  ,  D2C( 168 ) )
cp850tab ~ PUT( "&reg;"     ,  D2C( 169 ) )
cp850tab ~ PUT( "&not;"     ,  D2C( 170 ) )
cp850tab ~ PUT( "&frac12;"  ,  D2C( 171 ) )
cp850tab ~ PUT( "&frac14;"  ,  D2C( 172 ) )
cp850tab ~ PUT( "&iexcl;"   ,  D2C( 173 ) )
cp850tab ~ PUT( "&laquo;"   ,  D2C( 174 ) )
cp850tab ~ PUT( "&raquo;"   ,  D2C( 175 ) )
cp850tab ~ PUT( "&Aacute;"  ,  D2C( 181 ) )
cp850tab ~ PUT( "&Acirc;"   ,  D2C( 182 ) )
cp850tab ~ PUT( "&Agrave;"  ,  D2C( 183 ) )
cp850tab ~ PUT( "&copy;"    ,  D2C( 184 ) )
cp850tab ~ PUT( "&cent;"    ,  D2C( 189 ) )
cp850tab ~ PUT( "&yen;"     ,  D2C( 190 ) )
cp850tab ~ PUT( "&atilde;"  ,  D2C( 198 ) )
cp850tab ~ PUT( "&Atilde;"  ,  D2C( 199 ) )
cp850tab ~ PUT( "&curren;"  ,  D2C( 207 ) )
cp850tab ~ PUT( "&eth;"     ,  D2C( 208 ) )
cp850tab ~ PUT( "&ETH;"     ,  D2C( 209 ) )
cp850tab ~ PUT( "&Ecirc;"   ,  D2C( 210 ) )
cp850tab ~ PUT( "&Euml;"    ,  D2C( 211 ) )
cp850tab ~ PUT( "&Egrave;"  ,  D2C( 212 ) )
cp850tab ~ PUT( "&Iacute;"  ,  D2C( 214 ) )
cp850tab ~ PUT( "&Icirc;"   ,  D2C( 215 ) )
cp850tab ~ PUT( "&Iuml;"    ,  D2C( 216 ) )
cp850tab ~ PUT( "&brkbar;"  ,  D2C( 221 ) )
cp850tab ~ PUT( "&Igrave;"  ,  D2C( 222 ) )
cp850tab ~ PUT( "&Oacute;"  ,  D2C( 224 ) )
cp850tab ~ PUT( "&szlig;"   ,  D2C( 225 ) )
cp850tab ~ PUT( "&Ocirc;"   ,  D2C( 226 ) )
cp850tab ~ PUT( "&Ograve;"  ,  D2C( 227 ) )
cp850tab ~ PUT( "&otilde;"  ,  D2C( 228 ) )
cp850tab ~ PUT( "&Otilde;"  ,  D2C( 229 ) )
cp850tab ~ PUT( "&micro;"   ,  D2C( 230 ) )
cp850tab ~ PUT( "&THORN;"   ,  D2C( 231 ) )
cp850tab ~ PUT( "&thorn;"   ,  D2C( 232 ) )
cp850tab ~ PUT( "&Uacute;"  ,  D2C( 233 ) )
cp850tab ~ PUT( "&Ucirc;"   ,  D2C( 234 ) )
cp850tab ~ PUT( "&Ugrave;"  ,  D2C( 235 ) )
cp850tab ~ PUT( "&yacute;"  ,  D2C( 236 ) )
cp850tab ~ PUT( "&Yacute;"  ,  D2C( 237 ) )
cp850tab ~ PUT( "&hibar;"   ,  D2C( 238 ) )
cp850tab ~ PUT( "&acute;"   ,  D2C( 239 ) )
cp850tab ~ PUT( "&plusmn;"  ,  D2C( 241 ) )
cp850tab ~ PUT( "&frac34;"  ,  D2C( 243 ) )
cp850tab ~ PUT( "&para;"    ,  D2C( 244 ) )
cp850tab ~ PUT( "&sect;"    ,  D2C( 245 ) )
cp850tab ~ PUT( "&divide;"  ,  D2C( 246 ) )
cp850tab ~ PUT( "&cedil;"   ,  D2C( 247 ) )
cp850tab ~ PUT( "&deg;"     ,  D2C( 248 ) )
cp850tab ~ PUT( "&uml;"     ,  D2C( 249 ) )
cp850tab ~ PUT( "&middot;"  ,  D2C( 250 ) )
cp850tab ~ PUT( "&sup1;"    ,  D2C( 251 ) )
cp850tab ~ PUT( "&sup3;"    ,  D2C( 252 ) )
cp850tab ~ PUT( "&sup2;"    ,  D2C( 253 ) )

.SGMLEntity ~ setentry(    "850", cp850tab ) /* add codepage table to directory */
.SGMLEntity ~ setentry( "ibm850", cp850tab ) /* add codepage table to directory */
.SGMLEntity ~ setentry(  "850de", cp850tab ) /* add codepage table to directory */



/* Codepage 437 (US) , almost the same as 850, except ... */

cp437tab = cp850tab ~ copy      /* make a copy of cp850tab */

/* the following entries differ from 850, replace them: */

cp437tab ~ PUT( "&cent;"    ,  D2C( 155 ) )
cp437tab ~ PUT( "&yen;"     ,  D2C( 157 ) )

/* remove entries in code page "437" which do not have a corresponding SGML-entity,
   i.e. leave them verbatim in the text */

cp437tab ~ REMOVE( D2C( 158 )  )
cp437tab ~ REMOVE( D2C( 169 )  )
cp437tab ~ REMOVE( D2C( 181 )  )
cp437tab ~ REMOVE( D2C( 182 )  )
cp437tab ~ REMOVE( D2C( 183 )  )
cp437tab ~ REMOVE( D2C( 184 )  )
cp437tab ~ REMOVE( D2C( 189 )  )
cp437tab ~ REMOVE( D2C( 190 )  )
cp437tab ~ REMOVE( D2C( 198 )  )
cp437tab ~ REMOVE( D2C( 199 )  )
cp437tab ~ REMOVE( D2C( 208 )  )
cp437tab ~ REMOVE( D2C( 209 )  )
cp437tab ~ REMOVE( D2C( 210 )  )
cp437tab ~ REMOVE( D2C( 211 )  )
cp437tab ~ REMOVE( D2C( 212 )  )
cp437tab ~ REMOVE( D2C( 214 )  )
cp437tab ~ REMOVE( D2C( 215 )  )
cp437tab ~ REMOVE( D2C( 216 )  )
cp437tab ~ REMOVE( D2C( 221 )  )
cp437tab ~ REMOVE( D2C( 222 )  )
cp437tab ~ REMOVE( D2C( 224 )  )
cp437tab ~ REMOVE( D2C( 225 )  )
cp437tab ~ REMOVE( D2C( 226 )  )
cp437tab ~ REMOVE( D2C( 227 )  )
cp437tab ~ REMOVE( D2C( 228 )  )
cp437tab ~ REMOVE( D2C( 229 )  )
cp437tab ~ REMOVE( D2C( 231 )  )
cp437tab ~ REMOVE( D2C( 232 )  )
cp437tab ~ REMOVE( D2C( 233 )  )
cp437tab ~ REMOVE( D2C( 234 )  )
cp437tab ~ REMOVE( D2C( 235 )  )
cp437tab ~ REMOVE( D2C( 236 )  )
cp437tab ~ REMOVE( D2C( 237 )  )
cp437tab ~ REMOVE( D2C( 238 )  )
cp437tab ~ REMOVE( D2C( 239 )  )
cp437tab ~ REMOVE( D2C( 243 )  )
cp437tab ~ REMOVE( D2C( 244 )  )
cp437tab ~ REMOVE( D2C( 245 )  )
cp437tab ~ REMOVE( D2C( 247 )  )
cp437tab ~ REMOVE( D2C( 249 )  )
cp437tab ~ REMOVE( D2C( 251 )  )
cp437tab ~ REMOVE( D2C( 252 )  )

.SGMLEntity ~ setentry(    "437", cp437tab ) /* add codepage table to directory */
.SGMLEntity ~ setentry( "ibm437", cp437tab ) /* add codepage table to directory */
.SGMLEntity ~ setentry(  "437us", cp437tab ) /* add codepage table to directory */


/* 19990111 13:56:53 ---rgf, 99-01-11, wuw */

cpLatin1 = .table ~ new    /* iso-8859-1 a.k.a. 'latin1' */
                                                    /* Unicode names:                      */
cpLatin1 ~ PUT( "&nbsp;"             , D2C( 160 ) ) /* no-break space                      */
cpLatin1 ~ PUT( "&iexcl;"            , D2C( 161 ) ) /* inverted exclamation mark           */
cpLatin1 ~ PUT( "&cent;"             , D2C( 162 ) ) /* cent sign                           */
cpLatin1 ~ PUT( "&pound;"            , D2C( 163 ) ) /* pound sterling sign                 */
cpLatin1 ~ PUT( "&curren;"           , D2C( 164 ) ) /* general currency sign               */
cpLatin1 ~ PUT( "&yen;"              , D2C( 165 ) ) /* yen sign                            */
cpLatin1 ~ PUT( "&brvbar;"           , D2C( 166 ) ) /* broken (vertical) bar               */
cpLatin1 ~ PUT( "&sect;"             , D2C( 167 ) ) /* section sign                        */
cpLatin1 ~ PUT( "&uml;"              , D2C( 168 ) ) /* umlaut (dieresis)                   */
cpLatin1 ~ PUT( "&copy;"             , D2C( 169 ) ) /* copyright sign                      */
cpLatin1 ~ PUT( "&ordf;"             , D2C( 170 ) ) /* ordinal indicator, feminine         */
cpLatin1 ~ PUT( "&laquo;"            , D2C( 171 ) ) /* angle quotation mark, left          */
cpLatin1 ~ PUT( "&not;"              , D2C( 172 ) ) /* not sign                            */
cpLatin1 ~ PUT( "&shy;"              , D2C( 173 ) ) /* soft hyphen                         */
cpLatin1 ~ PUT( "&reg;"              , D2C( 174 ) ) /* registered sign                     */
cpLatin1 ~ PUT( "&macr;"             , D2C( 175 ) ) /* macron                              */
cpLatin1 ~ PUT( "&deg;"              , D2C( 176 ) ) /* degree sign                         */
cpLatin1 ~ PUT( "&plusmn;"           , D2C( 177 ) ) /* plus-or-minus sign                  */
cpLatin1 ~ PUT( "&sup2;"             , D2C( 178 ) ) /* superscript two                     */
cpLatin1 ~ PUT( "&sup3;"             , D2C( 179 ) ) /* superscript three                   */
cpLatin1 ~ PUT( "&acute;"            , D2C( 180 ) ) /* acute accent                        */
cpLatin1 ~ PUT( "&micro;"            , D2C( 181 ) ) /* micro sign                          */
cpLatin1 ~ PUT( "&para;"             , D2C( 182 ) ) /* pilcrow (paragraph sign)            */
cpLatin1 ~ PUT( "&middot;"           , D2C( 183 ) ) /* middle dot                          */
cpLatin1 ~ PUT( "&cedil;"            , D2C( 184 ) ) /* cedilla                             */
cpLatin1 ~ PUT( "&sup1;"             , D2C( 185 ) ) /* superscript one                     */
cpLatin1 ~ PUT( "&ordm;"             , D2C( 186 ) ) /* ordinal indicator, masculine        */
cpLatin1 ~ PUT( "&raquo;"            , D2C( 187 ) ) /* angle quotation mark, right         */
cpLatin1 ~ PUT( "&frac14;"           , D2C( 188 ) ) /* fraction one-quarter                */
cpLatin1 ~ PUT( "&frac12;"           , D2C( 189 ) ) /* fraction one-half                   */
cpLatin1 ~ PUT( "&frac34;"           , D2C( 190 ) ) /* fraction three-quarters             */
cpLatin1 ~ PUT( "&iquest;"           , D2C( 191 ) ) /* inverted question mark              */
cpLatin1 ~ PUT( "&Agrave;"           , D2C( 192 ) ) /* capital A, grave accent             */
cpLatin1 ~ PUT( "&Aacute;"           , D2C( 193 ) ) /* capital A, acute accent             */
cpLatin1 ~ PUT( "&Acirc;"            , D2C( 194 ) ) /* capital A, circumflex accent        */
cpLatin1 ~ PUT( "&Atilde;"           , D2C( 195 ) ) /* capital A, tilde                    */
cpLatin1 ~ PUT( "&Auml;"             , D2C( 196 ) ) /* capital A, dieresis or umlaut mark  */
cpLatin1 ~ PUT( "&Aring;"            , D2C( 197 ) ) /* capital A, ring                     */
cpLatin1 ~ PUT( "&AElig;"            , D2C( 198 ) ) /* capital AE diphthong (ligature)     */
cpLatin1 ~ PUT( "&Ccedil;"           , D2C( 199 ) ) /* capital C, cedilla                  */
cpLatin1 ~ PUT( "&Egrave;"           , D2C( 200 ) ) /* capital E, grave accent             */
cpLatin1 ~ PUT( "&Eacute;"           , D2C( 201 ) ) /* capital E, acute accent             */
cpLatin1 ~ PUT( "&Ecirc;"            , D2C( 202 ) ) /* capital E, circumflex accent        */
cpLatin1 ~ PUT( "&Euml;"             , D2C( 203 ) ) /* capital E, dieresis or umlaut mark  */
cpLatin1 ~ PUT( "&Igrave;"           , D2C( 204 ) ) /* capital I, grave accent             */
cpLatin1 ~ PUT( "&Iacute;"           , D2C( 205 ) ) /* capital I, acute accent             */
cpLatin1 ~ PUT( "&Icirc;"            , D2C( 206 ) ) /* capital I, circumflex accent        */
cpLatin1 ~ PUT( "&Iuml;"             , D2C( 207 ) ) /* capital I, dieresis or umlaut mark  */
cpLatin1 ~ PUT( "&ETH;"              , D2C( 208 ) ) /* capital Eth, Icelandic              */
cpLatin1 ~ PUT( "&Ntilde;"           , D2C( 209 ) ) /* capital N, tilde                    */
cpLatin1 ~ PUT( "&Ograve;"           , D2C( 210 ) ) /* capital O, grave accent             */
cpLatin1 ~ PUT( "&Oacute;"           , D2C( 211 ) ) /* capital O, acute accent             */
cpLatin1 ~ PUT( "&Ocirc;"            , D2C( 212 ) ) /* capital O, circumflex accent        */
cpLatin1 ~ PUT( "&Otilde;"           , D2C( 213 ) ) /* capital O, tilde                    */
cpLatin1 ~ PUT( "&Ouml;"             , D2C( 214 ) ) /* capital O, dieresis or umlaut mark  */
cpLatin1 ~ PUT( "&times;"            , D2C( 215 ) ) /* multiply sign                       */
cpLatin1 ~ PUT( "&Oslash;"           , D2C( 216 ) ) /* capital O, slash                    */
cpLatin1 ~ PUT( "&Ugrave;"           , D2C( 217 ) ) /* capital U, grave accent             */
cpLatin1 ~ PUT( "&Uacute;"           , D2C( 218 ) ) /* capital U, acute accent             */
cpLatin1 ~ PUT( "&Ucirc;"            , D2C( 219 ) ) /* capital U, circumflex accent        */
cpLatin1 ~ PUT( "&Uuml;"             , D2C( 220 ) ) /* capital U, dieresis or umlaut mark  */
cpLatin1 ~ PUT( "&Yacute;"           , D2C( 221 ) ) /* capital Y, acute accent             */
cpLatin1 ~ PUT( "&THORN;"            , D2C( 222 ) ) /* capital THORN, Icelandic            */
cpLatin1 ~ PUT( "&szlig;"            , D2C( 223 ) ) /* small sharp s, German (sz ligature) */
cpLatin1 ~ PUT( "&agrave;"           , D2C( 224 ) ) /* small a, grave accent               */
cpLatin1 ~ PUT( "&aacute;"           , D2C( 225 ) ) /* small a, acute accent               */
cpLatin1 ~ PUT( "&acirc;"            , D2C( 226 ) ) /* small a, circumflex accent          */
cpLatin1 ~ PUT( "&atilde;"           , D2C( 227 ) ) /* small a, tilde                      */
cpLatin1 ~ PUT( "&auml;"             , D2C( 228 ) ) /* small a, dieresis or umlaut mark    */
cpLatin1 ~ PUT( "&aring;"            , D2C( 229 ) ) /* small a, ring                       */
cpLatin1 ~ PUT( "&aelig;"            , D2C( 230 ) ) /* small ae diphthong (ligature)       */
cpLatin1 ~ PUT( "&ccedil;"           , D2C( 231 ) ) /* small c, cedilla                    */
cpLatin1 ~ PUT( "&egrave;"           , D2C( 232 ) ) /* small e, grave accent               */
cpLatin1 ~ PUT( "&eacute;"           , D2C( 233 ) ) /* small e, acute accent               */
cpLatin1 ~ PUT( "&ecirc;"            , D2C( 234 ) ) /* small e, circumflex accent          */
cpLatin1 ~ PUT( "&euml;"             , D2C( 235 ) ) /* small e, dieresis or umlaut mark    */
cpLatin1 ~ PUT( "&igrave;"           , D2C( 236 ) ) /* small i, grave accent               */
cpLatin1 ~ PUT( "&iacute;"           , D2C( 237 ) ) /* small i, acute accent               */
cpLatin1 ~ PUT( "&icirc;"            , D2C( 238 ) ) /* small i, circumflex accent          */
cpLatin1 ~ PUT( "&iuml;"             , D2C( 239 ) ) /* small i, dieresis or umlaut mark    */
cpLatin1 ~ PUT( "&eth;"              , D2C( 240 ) ) /* small eth, Icelandic                */
cpLatin1 ~ PUT( "&ntilde;"           , D2C( 241 ) ) /* small n, tilde                      */
cpLatin1 ~ PUT( "&ograve;"           , D2C( 242 ) ) /* small o, grave accent               */
cpLatin1 ~ PUT( "&oacute;"           , D2C( 243 ) ) /* small o, acute accent               */
cpLatin1 ~ PUT( "&ocirc;"            , D2C( 244 ) ) /* small o, circumflex accent          */
cpLatin1 ~ PUT( "&otilde;"           , D2C( 245 ) ) /* small o, tilde                      */
cpLatin1 ~ PUT( "&ouml;"             , D2C( 246 ) ) /* small o, dieresis or umlaut mark    */
cpLatin1 ~ PUT( "&divide;"           , D2C( 247 ) ) /* divide sign                         */
cpLatin1 ~ PUT( "&oslash;"           , D2C( 248 ) ) /* small o, slash                      */
cpLatin1 ~ PUT( "&ugrave;"           , D2C( 249 ) ) /* small u, grave accent               */
cpLatin1 ~ PUT( "&uacute;"           , D2C( 250 ) ) /* small u, acute accent               */
cpLatin1 ~ PUT( "&ucirc;"            , D2C( 251 ) ) /* small u, circumflex accent          */
cpLatin1 ~ PUT( "&uuml;"             , D2C( 252 ) ) /* small u, dieresis or umlaut mark    */
cpLatin1 ~ PUT( "&yacute;"           , D2C( 253 ) ) /* small y, acute accent               */
cpLatin1 ~ PUT( "&thorn;"            , D2C( 254 ) ) /* small thorn, Icelandic              */
cpLatin1 ~ PUT( "&yuml;"             , D2C( 255 ) ) /* small y, dieresis or umlaut mark    */

.SGMLEntity ~ setentry(        "819", cpLatin1 )
.SGMLEntity ~ setentry(     "ibm819", cpLatin1 )
.SGMLEntity ~ setentry(  "iso8859-1", cpLatin1 )
.SGMLEntity ~ setentry( "iso_8859-1", cpLatin1 )
.SGMLEntity ~ setentry(   "iso88591", cpLatin1 )
.SGMLEntity ~ setentry(     "latin1", cpLatin1 )



/* define default SGMLEntity-table, if codepage is 0 */

processCodePage = "latin1"                              /* assuming non-OS/2    */
If RxFuncQuery( "SysQueryProcessCodePage" ) = 0 THEN    /* "0", if available    */
   processCodePage = SysQueryProcessCodePage()          /* get codepage of current process      */

IF .SGMLEntity ~ hasentry( processCodePage ) THEN       /* use process' CP as default CP*/
   .SGMLEntity ~ setentry( 0, .SGMLEntity ~ entry( processCodePage) )
ELSE                                                    /* not found            */
   .SGMLEntity ~ setentry( 0, "latin1" )                /* default to latin1-codepage   */




:: REQUIRES nls_util.rex        /* load national language support */

/* ---------------------------------------------------------------------------
CPFile2SGMLEntity( filename [, [CodePage] [, "Reverse" ] ] )
CPString2SGMLEntity( string [, [CodePage] [, "Reverse" ] ] )

  e.g. CPString2SGMLEntity( "h&ouml;her", "850", "Reverse"  ) ... change string back, if possible
         CPFile2SGMLEntity( "d:\aha.html", "850" )            ... change a file

--------------------------------------------------------------------------- */
/* work on a file                       */
:: ROUTINE CPFile2SGMLEntity    PUBLIC
   USE ARG filename, CodePage, Reverse

   IF \ VAR( "CodePage" ) THEN CodePage = get_nls_default_object() ~ codepage
   IF \ VAR( "Reverse" )  THEN Reverse  = ""

   /* read file into one variable, to treat it as a large string */
   stream = .stream ~ new( filename ) ~~ open       /* open for reading and writing */
   FileString = stream ~ charin( 1, stream ~ query( "Size" ) )
   stream ~ close
   FileString = CPString2SGMLEntity( FileString, CodePage, Reverse )
   stream ~~  open( "WRITE REPLACE" )           /* truncate file                */
   stream ~ charout( FileString, 1 )            /* write string back to file    */
   stream ~ close
   DROP FileString




/* work on a string                     */
:: ROUTINE CPString2SGMLEntity  PUBLIC
   USE ARG string, CodePage, Reverse

   IF \ VAR( "CodePage" ) THEN CodePage = get_nls_default_object() ~ codepage
   IF \ VAR( "Reverse" )  THEN Reverse  = ""

                                /* does given CP exist, if not use default CP   */
   cpTable = .SGMLEntity ~ entry( codepage )    /* try to retrieve CP-2-SGMLentity table  */
   IF cpTable = .nil THEN       /* SGMLentity-table for codepage not found, raise error */
   DO
      SIGNAL ON SYNTAX
      RAISE SYNTAX 40.904 ARRAY ( "CPString2SGMLEntity()", '2 ("CodePage")',,
                                  "supported SGMLEntity codepages", codepage )
   END
                                /* call proc according to third argument        */
   IF ( TRANSLATE( LEFT( Reverse, 1 ) ) = "R" ) THEN
      RETURN sgml2chars( string )       /* reverse      */
   ELSE
      RETURN chars2sgml( string )

/* --------------- procedure ----------------- */
CHARS2SGML :
   tmpSupp = cpTable ~ supplier /* get a supplier from table */
   DO WHILE tmpSupp ~ available
      old  = tmpSupp ~ index    /* "needle"     */
      new  = tmpSupp ~ item     /* "newneedle"  */

      IF POS( old, string ) > 0 THEN            /* make as few new string-objects as possible!  */
         string = CHANGESTR( old, string, new ) /* translate chars to HTML/SGML entities */

      tmpSupp ~ next            /* get next item */
   END
   RETURN string


/* --------------- procedure ----------------- */
SGML2CHARS :
   tmpSupp = cpTable ~ supplier /* get a supplier from table */
   DO WHILE tmpSupp ~ available
      old  = tmpSupp ~ item     /* "needle"     */
      new  = tmpSupp ~ index    /* "newneedle"  */

      IF POS( old, string ) > 0 THEN            /* make as few new string-objects as possible!  */
         string = CHANGESTR( old, string, new ) /* translate chars to HTML/SGML entities */

      tmpSupp ~ next            /* get next item */
   END
   RETURN string


SYNTAX : RAISE PROPAGATE        /* raise error in caller        */




/* ========================================================================= */

/* this class keeps known codepages <---> HTML translation tables at the class level,
   if the class is asked e.g. for codepages 850 or 437, then the table describing them will be
   returned;

   one can add any other table by adding (setentry), replacing (setentry) or removing (setentry)
   it at the class level ...
   (the UNKNOWN class method will forward it to the class object directory-variable )

*/

:: CLASS SGMLEntity           /* PUBLIC */

:: METHOD init CLASS
   EXPOSE cpDirectory

   cpDirectory = .directory ~ new       /* directory to contain codepages */



:: METHOD unknown       CLASS           /* forward messages to .directory */
   EXPOSE cpDirectory
   USE ARG mess_name, mess_args

   FORWARD MESSAGE (mess_name) ARGUMENTS (mess_args) TO ( cpDirectory )

